/*******************************************************************
*
*      Copyright (c) 1994-1999 Jetico, Inc., Finland
*      All rights reserved.
*
*      File: bccreate.c
*      Revision: $Id: bcmount.c,v 1.33 2005/05/14 07:58:33 crypt Rel-1.6-3 $
*      Created:
*      Description: implementation of mount container routine
*
*******************************************************************/



#include <sys/stat.h>
#include <stdlib.h>
#include <dirent.h>
#include <errno.h>
#include <stdio.h>
#include <mntent.h>
#include <unistd.h>
#include <sys/mount.h>
#ifndef MS_MGC_VAL
    #include <linux/fs.h>
#endif
#include <bc_types.h>
#include <bc_ioctl.h>
#include "keygen.h"
#include <kg_defs.h>
#include <fcntl.h>
#include <kblock.h>
#include <string.h>
#include "alg.h"
#include "header.h"
#include "config.h"
#include "bctool.h"
#include "misc.h"
#include "error.h"

char bcmount_c[]="$Id: bcmount.c,v 1.33 2005/05/14 07:58:33 crypt Rel-1.6-3 $";

int bcumount_all (int argc, char ** argv);

int local_umount(char *mount_point)
{
    char *device;
    ALG_SERV Alg;
    int Error;

    sync();

    device=FindMountedDevice(mount_point);
    if ( device==NULL )
    {
        msg(stderr,"Can not find mounted device\n");
        return -1;
    }

    if ( (Alg.bc_fd = open(device, O_RDONLY)) < 0 )
    {
        msg(stderr,"open(%s,O_RDONLY): %s\n",device,strerror(errno));
        return -1;
    }

    if ( bc_get_priv(Alg,0)!=0 )
    {
        msg(stderr,"bc_get_priv: %s\n",strerror(errno));
        close(Alg.bc_fd);
        return -1;
    }
    
    if (o_force)
    {
        Error=bc_force_unlock(Alg);
        if ( Error!=0 )
        {
            msg(stderr,"bc_force_unlock(%s): %s\n",device,strerror(-Error));
            close(Alg.bc_fd);
            return -1;
        }
    }else
    {
        Error=bc_unlock_dev(Alg);
        if ( Error!=0 )
        {
            msg(stderr,"bc_unlock_dev(%s): %s\n",device,strerror(-Error));
            close(Alg.bc_fd);
            return -1;
        }
    }

    if ( umount(mount_point)!=0 )
    {
        msg(stderr,"umount(%s): %s\n",mount_point,strerror(errno));
        bc_lock_dev(Alg);
        close(Alg.bc_fd);
        return -1;
    }
    
    Error=bc_clear_fd(Alg);
    if ( Error!=0 )
    {
        msg(stderr,"bc_clear_fd: %s\n",strerror(-Error));
        close(Alg.bc_fd);
        return -1;
    }
    close(Alg.bc_fd);
    return 0;
}

int bcmount (int argc, char ** argv)
{

    struct HiddenSector  hs;
    char *device;
    int fd, fdd, flag;
    long long Offset;
    ALG_SERV Alg;
    DWORD KeyHandle,DataSize,_offset;
    DATA_BLOCK db, *pdb;
    alg_info *alg_inf;

    FILE *mtab;
    struct mntent mnt;
    char rp1[NAME_MAX+PATH_MAX];
    char rp2[NAME_MAX+PATH_MAX];
    char options[256];
    unsigned long flags;
    int fs_id;



    if ( argc < 3 )
    {
        msg(stderr,"Please specify container name and mount point\n"); 
        return -1;
    }
    else if ( argc > 3 )
    {
        msg(stderr,"Too many parameters\n");
        return -1;
    }

    if ( (realpath(argv[1],rp1)) == NULL ) 
    {
        msg(stderr,"realpath(%s,rp1): %s\n",argv[1],strerror(errno));
        return -1;
    }
    
    if ( (realpath(argv[2],rp2)) == NULL )
    {
        msg(stderr,"realpath(%s,rp2): %s\n",argv[2],strerror(errno));
        return -1;
    }
    
    Error=IsOwner(ruid,argv[2]);
    if ( Error==FALSE && ruid!=0 )
    {
        msg(stderr,"Sorry, you are not owner of %s\n",rp2);
        return -1;
    }
    else if ( Error==-1 )
    {
        msg(stderr,"lstat(%s): %s\n",rp2,strerror(errno));
        return -1;
    }

    if ( (device=FindFreeDevice())==NULL )
    {
        msg(stderr,"Can not find free device\n");
        return -1;
    }

    if ( (Alg.bc_fd = open(device, O_RDONLY)) < 0 )
    {
        msg(stderr,"open(%s): %s\n",device,strerror(errno));
        return -1;
    }
    if ( (fd = open(argv[1], O_RDONLY|O_LARGEFILE)) < 0 )
    {
        msg(stderr,"open(%s): %s\n",argv[1],strerror(errno));
        close(Alg.bc_fd);
        return -1; 
    }
    if ( read(fd, &hs, sizeof(hs))!=sizeof(hs) )
    {
        msg(stderr, "read(%s,&hs,%d): %s\n", argv[1], sizeof(hs), strerror(errno));
        close(Alg.bc_fd);
        close(fd);
        return -1; 
    }
    if ( strncmp(hs.br.OEMid, "LOCOS94 ", sizeof(hs.br.OEMid))!=0 )
    {
        msg(stderr,"%s is not valid BestCrypt container\n",argv[1]);
        close(Alg.bc_fd);
        close(fd);
        return -1;
    }   
    
    if (0 != access(argv[1],R_OK|W_OK))
    {
        msg(stderr,"You do not have write permissions for this container.\n \
            Mounting read-only\n");
        o_ro=1;
    }
    
    if ('B' == hs.br.jmpCode[0] && FALSE == o_ro)
    {
        msg(stderr,
            "WARNING! Container \"%s\" is probably in use!\n\n"
            "Two reasons for the report are possible:\n"
            "    1. The container is already mounted. In this case do not use the container\n"
            "    until someone dismounts it.\n"
            "    2. The container was not unmounted properly last time, so a lock mask \n"
            "    for the container was not reset. If so, unlock the container by running the command:\n"
            "        \"bctool unlock %s\" \n\n",
            argv[1],argv[1]);
        return -1;
    }
    if ( read(fd, &db, sizeof (db))!=sizeof(db) )
    {
        msg(stderr,"read(%s,&db,%d): %s\n",argv[1],sizeof(hs),strerror(errno));
        close(Alg.bc_fd);
        close(fd);
        return -1; 
    }
    if ( (alg_inf=GetInfoById(hs.algorithmId))==NULL )
    {
        msg(stderr,"Unknown algorithm Id: %d\n",hs.algorithmId);
        close(Alg.bc_fd);
        close(fd);
        return -1;
    }
    
    if ( o_fstype==NULL && (o_fstype=getFStypeById(hs.fileSystemId))==NULL )
    {
        msg(stderr,"Unknown filesystem. Try: -t fstype \n");
        return -1;
    }

    Alg.alg_id=hs.algorithmId;
    Alg.alg_module =alg_inf->mod_name;
    pdb = &db;
    DataSize = sizeof(db);
//    Offset=hs.dwDataOffset;

    if (KGSHA256_ID == hs.keyGenId)
    {
        UseSha256();
    }

    CreateKeyHandleEx(Alg, 
                      alg_inf->key_size, 
                      NULL, 
		      "Enter password:", 
		      CFLAG_VERIFY_AND_LOAD_KEY, 
		      (BYTE **)&pdb, 
		      &DataSize, 
		      &KeyHandle, 
		      &Error, 
		      &_offset);
    if ( Error!=0 )
    {
        print_error(Error);
        close(Alg.bc_fd);
        close(fd);
        return -1;
    }
    
    if (_offset)
        Offset = (long long)512 * _offset;
    else
        Offset = hs.dwDataOffset;

    Error=bc_set_fdkh64(Alg, 
			KeyHandle, 
			fd, 
			Offset, 0, 0,
			(o_ro ? BC_FLAGS_READONLY : 0) | 
			(hs.version == 0 ? 0 : BC_FLAGS_IV_64BIT));
    if ( Error!=0 )
    {
        msg(stderr,"bc_set_fd_kh: %s\n",strerror(-Error));
        FreeKeyHandle(Alg,KeyHandle);
        close(Alg.bc_fd);
        close(fd);
        return -1;
    }
//----------------------------------------------------------------------
// BestCrypt for Windows 7.11+ compatibility code
//----------------------------------------------------------------------    
    fdd = open(device, O_RDONLY);	// funny workaround for 2.4.x kernels
    flag = 0;
    _offset = check_msdos_partition(fdd < 0 ? Alg.bc_fd : fdd, &flag);
    if (fdd >= 0) close(fdd);
    
    if (0 != (flag & 0xFF000000)) { // partition table detected
        msg(stderr, "Warning! %d (%d extended) partition(s) detected inside container.\n",
		flag >> 24,
		(flag >> 16) & 0xFF);
	if (0x0100 != (flag >> 16)) { // non-trivial case
	    if (!ask_y_n("Do you want to continue? (y/[n]):")) {
		FreeKeyHandle(Alg, KeyHandle);
		close(Alg.bc_fd);
		close(fd);
		return 0;
	    }
	}
//	Offset += (long long) 512 * _offset;

	bc_clear_fd(Alg);

        Error=bc_set_fdkh64(Alg, 
			    KeyHandle,
			    fd, 
			    Offset, (long long)_offset, 0, 
			    (o_ro ? BC_FLAGS_READONLY : 0) | 
			    (hs.version == 0 ? 0 : BC_FLAGS_IV_64BIT));
        if ( Error!=0 )	{
	    msg(stderr,"bc_set_fd_kh: %s\n",strerror(-Error));
	    FreeKeyHandle(Alg,KeyHandle);
	    close(Alg.bc_fd);
	    close(fd);
	    return -1;
	}
    }
//----------------------------------------------------------------------    
//----------------------------------------------------------------------    
    
    
    snprintf(options,sizeof(options),"uid=%d,gid=%d,umask=%o",o_uid,o_gid,(~o_mode)&07777);
    flags=MS_MGC_VAL|MS_SYNCHRONOUS|(MS_RDONLY & o_ro);

    //
    // disable suid and dev files if not root 
    //
    if ( ruid!=0 )
        flags|=MS_NOSUID|MS_NODEV;

    fs_id=getIdByFStype(o_fstype);
    
    if ( bc_get_priv(Alg,0)!=0 )
    {
        msg(stderr,"bc_get_priv: %s\n",strerror(errno));
        bc_clear_fd(Alg);
        FreeKeyHandle(Alg,KeyHandle);
        close(Alg.bc_fd);
        close(fd);
        return -1;
    }


    if ( fs_id == 1 || fs_id == 2 || fs_id == 3 || fs_id == 8)
    {
        if ( mount(device,argv[2],o_fstype,flags,NULL)!=0 )
        {
            msg(stderr,"mount(%s,%s,%s,%X,NULL): %s\n",device,argv[2],o_fstype,flags,strerror(errno));
            bc_clear_fd(Alg);
            FreeKeyHandle(Alg,KeyHandle);
            close(Alg.bc_fd);
            close(fd);
            return -1;
        }
    }
    else if ( mount(device,argv[2],o_fstype,flags,options)!=0 &&
              mount(device,argv[2],o_fstype,flags,NULL)!=0 )
    {
        msg(stderr,"mount(%s,%s,%s,%X,%s): %s\n",device,argv[2],o_fstype,flags,options,strerror(errno));
        bc_clear_fd(Alg);
        FreeKeyHandle(Alg,KeyHandle);
        close(Alg.bc_fd);
        close(fd);
        return -1;
    }


    Error=bc_lock_dev(Alg);
    if ( Error!=0 )
    {
        msg(stderr,"bc_lock_dev(%s): %s\n",device,strerror(-Error));
        close(Alg.bc_fd);
        close(fd);
        return -1;
    }

    mnt.mnt_fsname = rp1;
    mnt.mnt_dir = rp2;
    mnt.mnt_type=o_fstype;
    mnt.mnt_opts= o_ro ? "ro,BestCrypt" : "rw,BestCrypt";
    mnt.mnt_freq=0;
    mnt.mnt_passno=0;
    mtab = setmntent("/etc/mtab",  "r+");
    if ( mtab != NULL )
    {
        addmntent(mtab, &mnt);
        endmntent(mtab);
    }
    else msg(stderr,"Can not open /etc/mtab. Mount entry will not be added in mtab file.\n");


    FreeKeyHandle(Alg,KeyHandle);

    close(fd);
    close(Alg.bc_fd);

    chown(rp2,o_uid,o_gid);
    chmod(rp2,o_mode);
    
    if ( setreuid(ruid,ruid)!=0 )
    {
        msg(stderr,"setreuid(%d,%d): %s\n",ruid,ruid,strerror(errno));
        return -1;
    }
    
    if (!o_ro && lock_container(argv[1]))
    {
        msg(stderr,"lock_container(\"%s\"): %s\n",argv[1],strerror(errno));
        return -1;
    }
    return 0;

}

int bcumount (int argc, char ** argv)
{

    ALG_SERV Alg;
    char *device;
    FILE *mtab,*tmptab;
    struct mntent *pmnt;
    char rp[NAME_MAX+PATH_MAX];

    if (o_umount_all)
    {
        return bcumount_all(argc,argv);
    }

    if ( argc < 2 )
    {
        msg(stderr,"Please specify mount point\n"); 
        return -1;
    }
    else if ( argc > 2 )
    {
        msg(stderr,"Too many parameters\n");
        return -1;
    }

    if ( NULL == realpath(argv[1],rp) )
    {
        msg(stderr,"realpath(%s,rp): %s\n",argv[1],strerror(errno));
        return -1;
    };

    Error=IsOwner(ruid,rp);
    if ( Error==FALSE && ruid!=0 )
    {
        msg(stderr,"sorry, you are not owner of this mount point\n");
        return -1;
    }
    else if ( Error==-1 )
    {
        msg(stderr,"IsOwner(%d,rp): %s\n",ruid,strerror(errno));
        return -1;
    }
    
    sync();

    device=FindMountedDevice(rp);
    if ( device==NULL )
    {
        msg(stderr,"Can not find mounted device\n");
        return -1;
    }

    if ( (Alg.bc_fd = open(device, O_RDONLY)) < 0 )
    {
        msg(stderr,"open(%s,O_RDONLY): %s\n",device,strerror(errno));
        return -1;
    }

    if ( bc_get_priv(Alg,0)!=0 )
    {
        msg(stderr,"bc_get_priv: %s\n",strerror(errno));
        close(Alg.bc_fd);
        return -1;
    }
    
    if (o_force)
    {
        Error=bc_force_unlock(Alg);
        if ( Error!=0 )
        {
            msg(stderr,"bc_force_unlock(%s): %s\n",device,strerror(-Error));
            close(Alg.bc_fd);
            return -1;
        }
    }else
    {
        Error=bc_unlock_dev(Alg);
        if ( Error!=0 )
        {
            msg(stderr,"bc_unlock_dev(%s): %s\n",device,strerror(-Error));
            close(Alg.bc_fd);
            return -1;
        }
    }

    if ( umount(rp)!=0 )
    {
        msg(stderr,"umount(%s): %s\n",rp,strerror(errno));
        bc_lock_dev(Alg);
        close(Alg.bc_fd);
        return -1;
    }

    mtab   = setmntent("/etc/mtab",  "r");
    tmptab = setmntent("/etc/mtab.tmp", "w+");
    if (NULL == mtab || NULL == tmptab)
    {
        msg(stderr,"Can not modify /etc/mtab file: %s\n",strerror(errno));
    }else 
    {
        struct stat st;

        while ( 1 )
        {
            pmnt=getmntent(mtab);
            if ( feof(mtab) )break;
            if ( strcmp(rp,pmnt->mnt_dir) )
                addmntent(tmptab, pmnt);
        }

        endmntent(mtab);
        endmntent(tmptab);

        if ( stat("/etc/mtab",&st)!=0 )
        {
            msg(stderr,"stat(\"/etc/mtab\"): %s\n",strerror(errno));
            st.st_mode = 0644;
            st.st_uid = 0;
            st.st_gid = 0;
        }
        
        if ( rename("/etc/mtab","/etc/mtab~")!=0 )
            msg(stderr,"rename(\"/etc/mtab\",\"/etc/mtab~\"): %s\n",strerror(errno));
        
        if ( rename("/etc/mtab.tmp","/etc/mtab")!=0 ) 
            msg(stderr,"rename(\"/etc/mtab.tmp\",\"/etc/mtab\"): %s\n",strerror(errno));
        
        if ( chown("/etc/mtab",st.st_uid,st.st_gid)!=0 ) 
            msg(stderr,"chown(\"/etc/mtab\",%d,%d): %s\n",st.st_uid,st.st_gid,strerror(errno));
        
        if ( chmod("/etc/mtab",st.st_mode)!=0 ) 
            msg(stderr,"chmod(\"/etc/mtab\",%o): %s\n",st.st_mode,strerror(errno));
        
        if ( unlink("/etc/mtab~")!=0 ) 
            msg(stderr,"unlink(\"/etc/mtab~\"): %s\n",strerror(errno));
    }

    if ( setreuid(ruid,ruid)!=0 )
    {
        msg(stderr,"setreuid(%d,%d): %s\n",ruid,ruid,strerror(errno));
        close(Alg.bc_fd);
        return -1;
    }
    
    Error=bc_clear_fd(Alg);
    if ( Error!=0 )
    {
        msg(stderr,"bc_clear_fd: %s\n",strerror(-Error));
        close(Alg.bc_fd);
        return -1;
    }


    close(Alg.bc_fd);
    
    return 0;

}

 
int bcumount_all (int argc, char ** argv)
{
    FILE *mtab,*tmptab;
    struct mntent *pmnt;
    struct stat st;
    int count = 0;
//    char tmptab_name[32];

    if ( argc > 1 )
    {
        msg(stderr,"Too many parameters.  ...ignoring...\n");
    }

    if ( get_priv(0)!=0 )
    {
        return -1;
    }
    
    mtab   = setmntent("/etc/mtab",  "r");
    tmptab = setmntent("/etc/mtab.tmp", "w+");
    
    if (NULL == mtab)
    {
        msg(stderr,"Can not read /etc/mtab file: %s\n",strerror(errno));
        return -1;
    }

    while ( 1 )
    {
        pmnt=getmntent(mtab);
        if ( feof(mtab) )break;

        if ( hasmntopt(pmnt, "BestCrypt") && (0 == ruid || IsOwner(ruid,pmnt->mnt_dir)) )
        {
            if (o_verbose) msg(stderr,"Unmounting \"%s\" ... \t",pmnt->mnt_dir);
            if (0 == local_umount(pmnt->mnt_dir))
            {
                count++;
                if (o_verbose) msg(stderr,"Done\n");
            }
        
        }else
        {
            if (tmptab)
                addmntent(tmptab, pmnt);
        }
    }

    endmntent(mtab);
    if (tmptab)
        endmntent(tmptab);

    if (tmptab && 0 != count)
    {
        if ( stat("/etc/mtab",&st)!=0 )
        {
            msg(stderr,"stat(\"/etc/mtab\"): %s\n",strerror(errno));
            st.st_mode = 0644;
            st.st_uid = 0;
            st.st_gid = 0;
        }

        if ( rename("/etc/mtab","/etc/mtab~")!=0 )
            msg(stderr,"rename(\"/etc/mtab\",\"/etc/mtab~\"): %s\n",strerror(errno));

        if ( rename("/etc/mtab.tmp","/etc/mtab")!=0 ) 
            msg(stderr,"rename(\"/etc/mtab.tmp\",\"/etc/mtab\"): %s\n",strerror(errno));

        if ( chown("/etc/mtab",st.st_uid,st.st_gid)!=0 ) 
            msg(stderr,"chown(\"/etc/mtab\",%d,%d): %s\n",st.st_uid,st.st_gid,strerror(errno));

        if ( chmod("/etc/mtab",st.st_mode)!=0 ) 
            msg(stderr,"chmod(\"/etc/mtab\",%o): %s\n",st.st_mode,strerror(errno));

        if ( unlink("/etc/mtab~")!=0 ) 
            msg(stderr,"unlink(\"/etc/mtab~\"): %s\n",strerror(errno));
    }
    
    unlink("/etc/mtab.tmp");
    if (o_verbose) msg(stderr,"Unmounted %d containers.\n",count);
    
    return 0;
}
