/*******************************************************************
*
*      Copyright (c) 1994-1999 Jetico, Inc., Finland
*      All rights reserved.
*
*      File: bccreate.c
*      Revision:    $Id: misc.c,v 1.36 2005/05/14 12:07:03 crypt Rel-1.6-3 $
*      Created:
*      Description: implementation of miscelleneous routines
*
*******************************************************************/



#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <unistd.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <fcntl.h>
#include <string.h>
#include <signal.h>
#include <stdarg.h>
#include <pwd.h>
#include <grp.h>
#include <assert.h>
#include <bc_types.h>
#include <bc_ioctl.h>
#include "keygen.h"
#include <kg_defs.h>
#include <fcntl.h>
//#include <kblock.h>
#include "header.h"
#include "alg.h"
#include "config.h"
#include "error.h"
#include "fstypes.h"
#include "misc.h"
#include "version.h"
#include <dlfcn.h>


char misc_c[]="$Id: misc.c,v 1.36 2005/05/14 12:07:03 crypt Rel-1.6-3 $";

#define MAX_DEVICES 16


char _device[MAX_DEV_PATH];

void SetSignalHandlers(void (*handler)(int))
{

    signal(SIGINT,handler);
    signal(SIGQUIT,handler);
    signal(SIGTERM,handler);
    signal(SIGABRT,handler);

}



char* strtolower(char *string)
{
    size_t i;
    for ( i=0;i<=strlen(string);i++ )string[i]=tolower(string[i]);
    return string;

}


void msg(FILE *fd, const char *format, ... )
{

    va_list args;

    va_start(args, format);
    vfprintf(fd, format, args);
    va_end(args);                                                                                                   

}

int is_valid_container(char *filename)
{

    struct HiddenSector  hs;
    int fd;


    if ( (fd = open(filename, O_RDONLY|O_LARGEFILE)) < 0 )
    {
        msg(stderr,"%s: %s\n",filename,strerror(errno));
        return FALSE;
    }
    if ( read(fd, &hs, sizeof(hs))!=sizeof(hs) )
    {
        msg(stderr,"read(\"%s\",&hs,sizeof(hs): %s\n",filename,strerror(errno));
        close(fd);
        return FALSE;
    }

    close(fd);

    if ( strncmp(hs.br.OEMid, "LOCOS94 ", sizeof(hs.br.OEMid))!=0 )
    {
        return FALSE;
    }

    if ( strncmp(hs.br.volumeLabel, "CRYPTED_DSK", sizeof(hs.br.volumeLabel))!=0 )
    {
        return FALSE;
    }

    return TRUE;

};




long long string_to_long(char *string)
{

    char *endptr;
    long long result;

    result=strtoul(string,&endptr,0);
    if ( string==endptr ) return 0;
    if ( (string[0]!='\0')&&(endptr[0]=='\0') ) return result;
    if ( (endptr[0]=='K' || endptr[0]=='k' )&&(endptr[1]=='\0') )
    {
        result *=1024;
        return result;
    };
    if ( (endptr[0]=='M' || endptr[0]=='m')&&(endptr[1]=='\0') )
    {
        result *=1024*1024;
        return result;
    };
    if ( (endptr[0]=='G' || endptr[0]=='g')&&(endptr[1]=='\0') )
    {
        result *=1024*1024*1024;
        return result;
    };
    return 0L;

}



int open_bc_dev(char *dev, int d)
{
    int fd;
    snprintf(dev,MAX_DEV_PATH,"/dev/bcrypt/disc%d/disc",d);
    if ( (fd = open(dev, O_RDONLY)) < 0 )
    {
        snprintf(dev,MAX_DEV_PATH,"/dev/bcrypt%d",d);
        fd = open(dev, O_RDONLY);
    }
    return fd;
}



char* FindFreeDevice()
{

    int i,n,rc;
    ALG_SERV Alg;

    for ( i=0;i<MAX_DEVICES;i++ )
    {
        Alg.bc_fd = open_bc_dev(_device, i);
        if ( Alg.bc_fd >= 0 )
        {
            rc=bc_get_info(Alg,&n,&n);
            close(Alg.bc_fd);
            if ( rc==0 ) return _device;
        }
    }

    return NULL;
}


char* FindMountedDevice(char *arg)
{

    struct stat buff1;
    int i;
    dev_t dev;

    if ( lstat(arg,&buff1)!=0 ) return NULL;
    if ( S_ISBLK(buff1.st_mode) ) return arg;
    dev=buff1.st_dev;

    for ( i=0;i<MAX_DEVICES;i++ )
    {
        snprintf(_device,MAX_DEV_PATH,"/dev/bcrypt/disc%d/disc",i);
        if ( lstat(_device,&buff1)==0 && dev==buff1.st_rdev ) return _device;
        snprintf(_device,MAX_DEV_PATH,"/dev/bcrypt%d",i);
        if ( lstat(_device,&buff1)==0 && dev==buff1.st_rdev ) return _device;
    }

    return NULL;
}


int IsOwner(uid_t uid, char *arg)
{

    struct stat buff;

    if ( lstat(arg,&buff)!=0 ) return -1;
    if ( buff.st_uid==uid ) return TRUE;
    return FALSE;

}   


int run_process(char *args[])
{
    int pid;
    char *env[] = { "PATH=/sbin", NULL};

    assert(args != NULL);
    assert(args[0] != NULL);

    pid = fork();
    if ( pid < 0 )
    {
        msg(stderr,"fork: %s\n", strerror(errno));
        return -1;
    }

    if ( pid == 0 )
    { /* Child */

        execve(args[0], args, env);
        msg(stderr,"execve %s failed: %s\n", args[0], strerror(errno));
    }

    /* Parent. */
    wait(NULL);
    return 0;
}

//ask_y_n(char *string)
//asks for yes or no
//return value:
//TRUE if 'y' entered
//FALSE in other cases
int ask_y_n(char *string)
{
    char c;
    msg(stderr,string);
    c = tolower(fgetc(stdin));
    if ( '\n' == c ) return FALSE;
    while ( '\n' != fgetc(stdin) );
    if ( 'y' == c ) return TRUE;
    return FALSE;
}

int get_uid_by_name(const char *name)
{
    struct passwd *pass;
    if ( NULL == name )
        return -1;
    pass=getpwnam(name);
    if ( NULL == pass )
        return -1;
    return pass->pw_uid;
}
int get_gid_by_name(const char *name)
{
    struct group *grp;
    if ( NULL == name )
        return -1;
    grp=getgrnam(name);
    if ( NULL == grp )
        return -1;
    return grp->gr_gid;

}


char* getFStypeById(unsigned int id)
{

    if ( id > MAX_ID )return NULL;
    return fstypes[id];
}

int   getIdByFStype(char *fstype)
{

    int i;

    for ( i=0;i<MAX_ID;i++ )if ( strcmp(fstype,fstypes[i])==0 ) return i;
    return MAX_ID + 1;

}

int modules_loaded()
{
    int major,minor;
    ALG_SERV Alg;
    char device[MAX_DEV_PATH];
    
    Alg.bc_fd = open_bc_dev(device, 0);

    if ( ENODEV == errno || 0 >  bc_get_info(Alg,&major,&minor) )
    {
        close(Alg.bc_fd);
        msg(stderr,"Kernel modules not loaded\n");
        return FALSE;
    }
    close(Alg.bc_fd);

    if (major != VERSION_MAJOR || minor != VERSION_MINOR)
    {
        msg(stderr,"One or more components of BestCrypt outdated.\n"
                   "driver version %d.%d\n"
                   "bctool version %d.%d\n"
                   "Please update your current version of BestCrypt\n",
            major,minor,VERSION_MAJOR,VERSION_MINOR);
        
        return FALSE;
    }

    return TRUE;
}


int lock_container(char *name)
{
    struct HiddenSector hs;
    int fd,len;

    if ( -1 == (fd  = open(name,O_RDWR|O_LARGEFILE)) )
    {
        return -1;   
    }
    len = read(fd,&hs,sizeof(hs)); 
    if ( -1 == len )
    {
        len = errno;
        close(fd);
        errno = len;
        return -1;
    }

    if ( lseek(fd,0,SEEK_SET) < 0 )
    {
        len = errno;
        close(fd);
        errno = len;
        return -1;
    }



    hs.br.jmpCode[0]='B';
    len = write(fd,&hs,sizeof(hs)); 
    if ( -1 == len )
    {
        len = errno;
        close(fd);
        errno = len;
        return -1;
    }

    close(fd);
    return 0;
}

int unlock_container(char *name)
{
    struct HiddenSector hs;
    int fd,len;

    if ( -1 == (fd  = open(name,O_RDWR|O_LARGEFILE)) )
    {
        return -1;   
    }
    len = read(fd,&hs,sizeof(hs)); 
    if ( -1 == len )
    {
        len = errno;
        close(fd);
        errno = len;
        return -1;
    }

    if ( lseek(fd,0,SEEK_SET) < 0 )
    {
        len = errno;
        close(fd);
        errno = len;
        return -1;
    }


    hs.br.jmpCode[0]=0xeb;
    len = write(fd,&hs,sizeof(hs)); 
    if ( -1 == len )
    {
        len = errno;
        close(fd);
        errno = len;
        return -1;
    }

    close(fd);
    return 0;                
}



int GetRandomSeed( char  *seed, int seedLength )
{

    FILE *fd;
    int  i;

    if ( ! (fd=fopen("/dev/urandom","r")) ) return ERROR_INTERNAL_PROBLEM;

    for ( i=0; i<seedLength; i++ )
        seed[i]=(char)fgetc(fd);

    fclose (fd);
    return ERROR_NO;
}                                                                                                                             

int get_priv(unsigned long drop_pid)
{
    ALG_SERV Alg;
    char device[MAX_DEV_PATH];


    Alg.bc_fd = open_bc_dev(device,0);

    if (-1 == Alg.bc_fd)
    {
        msg(stderr,"open(%s,O_RDONLY): %s\n",device, strerror(errno));
        close(Alg.bc_fd);
        return -1;
    }

    if ( bc_get_priv(Alg,drop_pid)!=0 )
    {
        msg(stderr,"bc_get_priv: %s\n",strerror(errno));
        close(Alg.bc_fd);
        return -1;
    }
    close(Alg.bc_fd);
    return 0;

}


void usage(FILE *fd)
{
    char *usage_string="Usage:\n\
bctool command [options] arguments\n\
\n\
commands:\n\
   new         - create container   (-a alg_name -s size [-d description] filename)\n\
   format      - format container   ([-t fstype] filename)\n\
   mount       - mount container    ([-t fstype] [-m mode] [-u user] [-g group] [-r] filename mountpoint)\n\
   umount      - unmount container  (mountpoint)\n\
   info        - get info on container (filename)\n\
   passwd      - change container's password (filename)\n\
   add_passwd  - add additional password for container (filename)\n\
   del_passwd  - delete additional password (filename)\n\
   fsck        - filesystem check (filename [-t fstype])\n\
   reencrypt   - reencrypt container (-a alg_name filename)\n\
   make_hidden - make hidden part in container (filename size_of_hidden_part)\n\
                 where: size_of_hidden_part may be xxx - in bytes; xxxK - in Kb; xxxM - in Mb; xx%% - in percents of container size\n\
   link        - link container to BestCrypt device (filename devicename)\n\
   unlink      - unlink container from device (devicename)\n\
\n\
options:\n\
   -t fstype   - to specify filesystem type (default: msdos)\n\
   -a alg_name - to specify algorithm name\n\
   -s size     - to specify container size\n\
        example: -s xxx - in bytes; -s xxxK - in Kb; -s xxxM - in Mb; -s xxxG - in Gb\n\
   -d desc     - to specify description of container (up to 64 symbols)\n\
   -r          - to mount read only\n\
   -m mode     - to specify mountpoint mode in octal (default 0700)\n\
   -u user     - to specify mountpoint's owner (root only can do it)\n\
   -g group    - to specify mountpoint's group\n\
\n";

    msg(fd,usage_string);

}

/*
int check_msdos_partition(int device_fd)

reads the first sector of block device;
returns first partition offset (sector) if 
partition table found, zero otherwise
*/

struct partition {
    unsigned char active;
    unsigned char start_head;
    unsigned char start_sec;
    unsigned char start_cyl;
    unsigned char os_type;
    unsigned char end_head;
    unsigned char end_sec;
    unsigned char end_cyl;
    unsigned int  lba_start_sec;
    unsigned int  lba_nr_sec;
} __attribute__((packed));

unsigned int check_msdos_partition(int device_fd, int *flags)
{
    struct partition	*prt;
    unsigned char	*buf;
    int			x[4], i, rc, flg;
    off_t		offset, sz;
    unsigned int	start;
    
    *flags = 0;
    buf = (unsigned char *)malloc(0x200);
    if (NULL == buf)
	return 0;

    sz = lseek64(device_fd, 0, SEEK_END);
    sz >>= 9; // device size in sectors
    
    // rewind to the very beginning
    offset = 0;
    offset = lseek64(device_fd, offset, SEEK_SET);
    if (0 != offset)
	goto err_out;
	
    // read the first sector of device
    // simply return 'no partition' on 
    // failure 
    rc = read(device_fd, buf, 0x200);
    if (0x200 != rc)
	goto err_out;
	
    // check msdos partition signature
    if (0x55 != buf[0x1FE] || 0xAA != buf[0x1FF])
	goto err_out;
	
    // partition entries offsets are
    // 0x1BE 0x1CE 0x1DE 0x1EE
    prt = (struct partition *)(buf+0x1BE);

    start = 0x0;
    flg   = 0x0000FFFF;
    for (i = 0; i < 4; i++, prt++) {
	x[i] = 0;
	if (0 == prt->lba_nr_sec)
	    continue; // no partition in this slot
	if (sz < prt->lba_nr_sec + prt->lba_start_sec)
	    goto err_out;
	    
	flg += 0x01000000;
	switch (prt->os_type) {
	case 0x05:	// DOS extended
	case 0x0F:	// Windows 95 extended
	case 0x85:	// Linux extended
	    x[i] = -prt->lba_start_sec;
	    flg += 0x00010000;
	    break;
	default:
	    x[i] = prt->lba_start_sec;
	    if (0 == start || start > prt->lba_start_sec) {
		start = prt->lba_start_sec;
		*flags &= 0xFFFF0000;
		*flags |= i;
	    }
	}
    }
    *flags = flg;
    return start;
err_out:
    if (NULL != buf)
	free(buf);    
    return 0;
}


static void *sha256handle;
static void *sha1handle;

static void* sha1CreateKeyHandle;
static void* sha256CreateKeyHandle;
static void* sha1CreateKeyHandleEx;
static void* sha256CreateKeyHandleEx;
static void* sha1FreeKeyHandle;
static void* sha256FreeKeyHandle;
static void* sha1FreeDataBlock;
static void* sha256FreeDataBlock;

/*
CreateKeyHandle
CreateKeyHandleEx
FreeKeyHandle
FreeDataBlock
*/
int LoadKeygens()
{
    char *error;


      sha1handle = dlopen ("libkgsha.so", RTLD_NOW);
      if (!sha1handle) {
          msg (stderr, "%s\n", dlerror());
          return -1;
      }

      sha256handle = dlopen ("libkgsha256.so", RTLD_NOW);
      if (!sha256handle) {
          msg (stderr, "%s\n", dlerror());
          return -1;
      }

      sha1CreateKeyHandle = dlsym(sha1handle, "CreateKeyHandle");
      if ((error = dlerror()) != NULL)  {
          msg (stderr, "%s\n", error);
          return -1;
      }
      sha1CreateKeyHandleEx = dlsym(sha1handle, "CreateKeyHandleEx");
      if ((error = dlerror()) != NULL)  {
          msg (stderr, "%s\n", error);
          return -1;
      }
      sha1FreeKeyHandle = dlsym(sha1handle, "FreeKeyHandle");
      if ((error = dlerror()) != NULL)  {
          msg (stderr, "%s\n", error);
          return -1;
      }
      sha1FreeDataBlock = dlsym(sha1handle, "FreeDataBlock");
      if ((error = dlerror()) != NULL)  {
          msg (stderr, "%s\n", error);
          return -1;
      }


      sha256CreateKeyHandle = dlsym(sha256handle, "CreateKeyHandle");
      if ((error = dlerror()) != NULL)  {
          msg (stderr, "%s\n", error);
          return -1;
      }
      sha256CreateKeyHandleEx = dlsym(sha256handle, "CreateKeyHandleEx");
      if ((error = dlerror()) != NULL)  {
          msg (stderr, "%s\n", error);
          return -1;
      }
      sha256FreeKeyHandle = dlsym(sha256handle, "FreeKeyHandle");
      if ((error = dlerror()) != NULL)  {
          msg (stderr, "%s\n", error);
          return -1;
      }
      sha256FreeDataBlock = dlsym(sha256handle, "FreeDataBlock");
      if ((error = dlerror()) != NULL)  {
          msg (stderr, "%s\n", error);
          return -1;
      }


      RandomGenerate = dlsym(sha1handle, "SHA1RandomGenerate");
      if ((error = dlerror()) != NULL)  {
          msg (stderr, "%s\n", error);
          return -1;
      }
      RandomGenerateKey = dlsym(sha1handle, "SHA1RandomGenerateKey");
      if ((error = dlerror()) != NULL)  {
          msg (stderr, "%s\n", error);
          return -1;
      }

      UseSha1();
      return 0;

}

int UnLoadKeygens()
{
    dlclose(sha1handle);
    dlclose(sha256handle);
    return 0;
}

int UseSha1()
{
    CreateKeyHandle     = sha1CreateKeyHandle;
    CreateKeyHandleEx   = sha1CreateKeyHandleEx;
    FreeKeyHandle       = sha1FreeKeyHandle;
    FreeDataBlock       = sha1FreeDataBlock;
    return 0;
}

int UseSha256()
{
    CreateKeyHandle     = sha256CreateKeyHandle;
    CreateKeyHandleEx   = sha256CreateKeyHandleEx;
    FreeKeyHandle       = sha256FreeKeyHandle;
    FreeDataBlock       = sha256FreeDataBlock;
    return 0;
}

